﻿using SyncFusionClientPaging.Shared;
using System;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Primitives;

namespace SyncFusionClientPaging.Server.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class WeatherForecastController : ControllerBase
    {
        private static readonly string[] Summaries = new[]
        {
            "Freezing", "Bracing", "Chilly", "Cool", "Mild", "Warm", "Balmy", "Hot", "Sweltering", "Scorching"
        };

        [HttpGet]
        public object Get()
        {
            // Create the random data
            var rng = new Random();
            var items = Enumerable.Range(1, 2000).Select(index => new WeatherForecast
            {
                Date = DateTime.Now.AddDays(index),
                TemperatureC = rng.Next(-20, 55),
                Summary = Summaries[rng.Next(Summaries.Length)]
            })
            .ToList();

            // Filter the data
            var count = items.Count();
            var queryString = Request.Query;
            if (queryString.Keys.Contains("$inlinecount"))
            {
                StringValues Skip;
                StringValues Take;
                StringValues OrderBy;

                int skip = (queryString.TryGetValue("$skip", out Skip)) ? Convert.ToInt32(Skip[0]) : 0;
                int top = (queryString.TryGetValue("$top", out Take)) ? Convert.ToInt32(Take[0]) : items.Count();
                string orderby = (queryString.TryGetValue("$orderby", out OrderBy)) ? OrderBy.ToString() : "Date";

                // Handle OrderBy direction
                if (orderby.EndsWith(" desc"))
                {
                    items = items.OrderByDescending(s => s.GetType().GetProperty(orderby.Replace(" desc", ""))
                    .GetValue(s)).ToList();
                }
                else
                {
                    items = items.OrderBy(s => s.GetType().GetProperty(orderby)
                    .GetValue(s)).ToList();
                }

                // Return result
                return new { Items = items.Skip(skip).Take(top), Count = count };
            }
            else
            {
                // No filter was passed
                // return first 10 items
                return items.Take(10);
            }
        }
    }
}
